/*
 * Program  to convert a sequence of a
 * taining file and some of a test file into a file 
 * that is appropriate for cn2
 *
 * Author: Shahzad Ali
 * @1999 CMU
 */

#include "defs.h"

static int createAttributeFile(int n, char *fn);
static int createExampleFile(int n, char* en, char* fn, char* tn, char* kn);

int main(int argc, char ** argv) {
  int c, num=0, len;
  char* trainfile= NULL, *testfile = NULL,*keyfile = NULL, 
    *examplefile = NULL, *attributefile = NULL;

  while ((c =getopt(argc, argv, "n:f:t:k:o:")) != -1){
    switch (c) {
    case 'n': {
      /* The number of attributes */
      num = atoi(optarg);
      break;
    }
    case 'f': {
      len = strlen(optarg);
      trainfile = new char[len + 1];
      if (!trainfile){
        cout << "Out of mem" << endl;
        exit(1);
      }
      strncpy(trainfile, optarg, len);
      trainfile[len] = '\0';
      break;
    }
    case 't': {
      len = strlen(optarg);
      testfile = new char[len + 1];
      if (!testfile){
        cout << "Out of mem" << endl;
        exit(1);
      }
      strncpy(testfile, optarg, len);
      testfile[len] = '\0';
      break;
    }
    case 'k': {
      len = strlen(optarg);
      keyfile = new char[len + 1];
      if (!keyfile){
        cout << "Out of mem" << endl;
        exit(1);
      }
      strncpy(keyfile, optarg, len);
      keyfile[len] = '\0';
      break;
    }
    case 'o': {
      len = strlen(optarg);
      /* Have to make two files from here */
      examplefile = new char[len + 4];
      attributefile = new char[len + 4];
      if (!examplefile || !attributefile){
        cout << "Out of mem" << endl;
        exit(1);
      }
      strncpy(examplefile, optarg, len);
      strncpy(attributefile, optarg, len);
      strcat(examplefile, ".exs");
      strcat(attributefile, ".att");
      examplefile[len+4] = '\0';
      attributefile[len+4] = '\0';
      break;
    }
    }
  }
  if (createAttributeFile(num, attributefile) < 0)
    return 1;
  if (createExampleFile(num, examplefile, trainfile, testfile, keyfile) < 0 )
    return 1;
}

static int
createAttributeFile(int n, char* fn){
  ofstream fout;

  if (!fn){
    cout << "Can not open attribute file" << endl;
    return (-1);
  }
  fout.open(fn);
  if (!fout.good()){
    cout << "Could not open attribute file %s" <<  fn << endl;
    return (-1);
  }
  if (n < 1){
    cout << "At least 1 attribute needed" << endl;
    return (-1);
  }
  fout << "**ATTRIBUTE FILE**" << endl << endl;
  for ( int i = 1; i <= n; i++)
    fout <<  "p" << i << ": " << " A B C D E F;" << endl;

  /*Last line for the class attribute */
  fout << "anomoly: y n;" << endl;
  fout.close();
  return (1);
}

static int 
createExampleFile(int n, char * en, char* fn, char* tn, char* kn) {
  ifstream fin, tin, kin;
  ofstream fout;
  char c, lastc;
  long lineno, minkey, maxkey;
  int flag=0;

  if (!en || !fn || !tn || !kn){
    cout << "One of the train, test or key files can not be opened" << endl;
    return (-1);
  }
  fin.open(fn);
  tin.open(tn);
  kin.open(kn);
  fout.open(en);
  if (!fout.good() || !fin.good() || !tin.good() || !kin.good()){
    cout << "One of the train, test or key files can not be opened" << endl;
    return (-1);
  }
  
  /* Set the header for the example file */
  fout << "**EXAMPLE FILE**" << endl << endl;
  fout << "% See the .att file for details of what these are" << endl;

  /* First get the training data in 
  * There is no anomoly in that data at least*/
  while (!fin.eof()){
    if (fin.eof())
      break;
    /* Write all the attribute values */
    for (int i=0; i<n; i++){
      fin >> c;
      if (fin.eof())
	c = lastc;
      fout << c << "\t";
      lastc = c;
    }
    /* Now the class */
    fout << "n;" << endl;
  }
  
  /* Now get some of the test data in.  We take TAKE_FROM_TEST 
   * records from the testfile. The keyfile has the places where
   * the anomolies are.
   */
  lineno = 0;
  /* Take the first key out */
  minkey = maxkey = -1;
  if (!kin.eof())
    kin >> minkey >> maxkey;

  while (!tin.eof()){
    int mark = 0;
    if (lineno > TAKE_FROM_TEST){
      if (!flag){
	fout.close();
	fout.open("real.exs");
	if (!fout.good()){
	  cout << "couldnot open real example file!" << endl;
	  return (-1);
	}
	fout << "**EXAMPLE FILE**" << endl << endl;
	fout << "% See the .att file for details of what these are" << endl; 
      }
      flag = 1;
    }
    if (tin.eof())
      break;
    if (kin.eof()){
      /* Set the key to -1 so that we know there is no key */
      minkey = maxkey = -1;
    }
    else if ((minkey != -1) && (lineno > maxkey)){
      /* Otherwise take the next key if lineno > key */
      kin >> minkey >> maxkey;
    }
    for ( int i = 0; i < n ; i++){
      tin >> c;
      lineno ++;
      if (!mark && (lineno >= minkey) && (lineno <= maxkey))
	mark = 1;
      fout << c << "\t";
    }
    if (mark)
      fout << "y;" << endl;
    else
      fout << "n;" << endl;
  }
  fin.close();
  tin.close();
  kin.close();
  fout.close();
  return 1;
}
